/////////////////////////////////////////////////////////////

// Defines for the ST7789 registers.
// ref: https://www.crystalfontz.com/controllers/Sitronix/ST7789V/
#define ST7789_00_NOP (0x00)
#define ST7789_01_SWRESET (0x01)
#define ST7789_04_RDDID (0x04)
#define ST7789_09_RDDST (0x09)
#define ST7789_0A_RDDPM (0x0A)
#define ST7789_0B_RDDMADCTL (0x0B)
#define ST7789_0C_RDDCOLMOD (0x0C)
#define ST7789_0D_RDDIM (0x0D)
#define ST7789_0E_RDDSM (0x0E)
#define ST7789_0F_RDDSDR (0x0F)
#define ST7789_10_SLPIN (0x10)
#define ST7789_11_SLPOUT (0x11)
#define ST7789_12_PTLON (0x12)
#define ST7789_13_NORON (0x13)
#define ST7789_20_INVOFF (0x20)
#define ST7789_21_INVON (0x21)
#define ST7789_26_GAMSET (0x26)
#define ST7789_28_DISPOFF (0x28)
#define ST7789_29_DISPON (0x29)
#define ST7789_2A_CASET (0x2A)
#define ST7789_2B_RASET (0x2B)
#define ST7789_2C_RAMWR (0x2C)
#define ST7789_2E_RAMRD (0x2E)
#define ST7789_30_PTLAR (0x30)
#define ST7789_33_VSCRDEF (0x33)
#define ST7789_34_TEOFF (0x34)
#define ST7789_35_TEON (0x35)
#define ST7789_36_MADCTL (0x36)
#define ST7789_37_VSCRSADD (0x37)
#define ST7789_38_IDMOFF (0x38)
#define ST7789_39_IDMON (0x39)
#define ST7789_3A_COLMOD (0x3A)
#define ST7789_3C_RAMWRC (0x3C)
#define ST7789_3E_RAMRDC (0x3E)
#define ST7789_44_TESCAN (0x44)
#define ST7789_45_RDTESCAN (0x45)
#define ST7789_51_WRDISBV (0x51)
#define ST7789_52_RDDISBV (0x52)
#define ST7789_53_WRCTRLD (0x53)
#define ST7789_54_RDCTRLD (0x54)
#define ST7789_55_WRCACE (0x55)
#define ST7789_56_RDCABC (0x56)
#define ST7789_5E_WRCABCMB (0x5E)
#define ST7789_5F_RDCABCMB (0x5F)
#define ST7789_68_RDABCSDR (0x68)
#define ST7789_B0_RAMCTRL (0xB0)
#define ST7789_B1_RGBCTRL (0xB1)
#define ST7789_B2_PORCTRL (0xB2)
#define ST7789_B3_FRCTRL1 (0xB3)
#define ST7789_B7_GCTRL (0xB7)
#define ST7789_BA_DGMEN (0xBA)
#define ST7789_BB_VCOMS (0xBB)
#define ST7789_C0_LCMCTRL (0xC0)
#define ST7789_C1_IDSET (0xC1)
#define ST7789_C2_VDVVRHEN (0xC2)
#define ST7789_C3_VRHS (0xC3)
#define ST7789_C4_VDVSET (0xC4)
#define ST7789_C5_VCMOFSET (0xC5)
#define ST7789_C6_FRCTR2 (0xC6)
#define ST7789_C7_CABCCTRL (0xC7)
#define ST7789_C8_REGSEL1 (0xC8)
#define ST7789_CA_REGSEL2 (0xCA)
#define ST7789_CC_PWMFRSEL (0xCC)
#define ST7789_D0_PWCTRL1 (0xD0)
#define ST7789_D2_VAPVANEN (0xD2)
#define ST7789_DA_RDID1 (0xDA)
#define ST7789_DB_RDID2 (0xDB)
#define ST7789_DC_RDID3 (0xDC)
#define ST7789_DF_CMD2EN (0xDF)
#define ST7789_E0_PVGAMCTRL (0xE0)
#define ST7789_E1_NVGAMCTRL (0xE1)
#define ST7789_E2_DGMLUTR (0xE2)
#define ST7789_E3_DGMLUTB (0xE3)
#define ST7789_E4_GATECTRL (0xE4)
#define ST7789_E8_PWCTRL2 (0xE8)
#define ST7789_E9_EQCTRL (0xE9)
#define ST7789_EC_PROMCTRL (0xEC)
#define ST7789_FA_PROMEN (0xFA)
#define ST7789_FC_NVMSET (0xFC)
#define ST7789_FE_PROMACT (0xFE)

//==============================================================================
inline void ST7789_sendCommand(uint8_t cmd)
{
	// Select the LCD controller
	LCD_CS_CLR;
	//Send the command via SPI:
	LCD_RS_CLR;
	ST7789_SPITransfer(cmd);
	// Deselect the LCD controller
	LCD_CS_SET;
}
//==============================================================================
inline void ST7789_sendData(uint8_t data)
{
	// Select the LCD controller
	LCD_CS_CLR;
	//Send the data via SPI:
	LCD_RS_SET;
	ST7789_SPITransfer(data);
	// Deselect the LCD controller
	LCD_CS_SET;
}

//==============================================================================
void ST7789_InitializeLCD(void)
{
	//Reset the LCD controller
	LCD_RES_CLR;
	delay(5); //10µS min
	LCD_RES_SET;
	delay(150); //120mS max

	// SLPOUT (11h): Sleep Out ("Sleep Out"  is chingrish for "wake")
	// The DC/DC converter is enabled, Internal display oscillator
	// is started, and panel scanning is started.
	ST7789_sendCommand(ST7789_11_SLPOUT);
	delay(120); //Delay 120ms

	// MADCTL (36h): Memory Data Access Control
	// Set the RGB vs BGR order to match a windows 24-bit BMP
	ST7789_sendCommand(ST7789_36_MADCTL);
	//ST7789_sendData(0x08 | 0x80); // YXVL RH--
	ST7789_sendData(0x08); // YXVL RH--
	// |||| ||||-- Unused: 0
	// |||| ||---- MH: Horizontal Refresh Order
	// |||| |        0 = left to right
	// |||| |        1 = right to left
	// |||| |----- RGB: RGB vs BGR Order
	// ||||          0 = RGB color filter panel
	// ||||          1 = BGR color filter panel
	// ||||------- ML: Vertical Refresh Order
	// |||           0 = top to bottom
	// |||           1 = bottom to top
	// |||-------- MV: Row / Column Exchange
	// ||--------- MX: Column Address Order  <<<<<
	// |---------- MY: Row Address Order

	// COLMOD (3Ah): Interface Pixel Format
	// * This command is used to define the format of RGB picture
	//   data, which is to be transferred via the MCU interface.
	ST7789_sendCommand(ST7789_3A_COLMOD);
	ST7789_sendData(0x06); // Default: 0x06 => 18-bit/pixel
						// IFPF[2:0] MCU Interface Color Format
						// IFPF[2:0] | Format
						//      000b | reserved
						//      001b | reserved
						//      010b | reserved
						//      011b | 12-bit/pixel
						//      100b | reserved
						//      101b | 16-bit/pixel
						//      110b | 18-bit/pixel   <<<<<
						//      111b | reserved

	// PORCTRL (B2h): Porch Setting
	ST7789_sendCommand(ST7789_B2_PORCTRL);
	ST7789_sendData(0x0C); //BPA[6:0]: Back porch setting in normal mode. The minimum setting is 0x01.
	ST7789_sendData(0x0C); //FPA[6:0]: Front porch setting in normal mode. The minimum setting is 0x01.
	ST7789_sendData(0x00); //Disable(0) / Enable (1) separate porch control
	ST7789_sendData(0x33); //(high nibble) BPB[3:0]: Back porch setting in idle mode. The minimum setting is 0x01.
						//(low nibble)  FPB[3:0]: Front porch setting in idle mode. The minimum setting is 0x01.
	ST7789_sendData(0x33); //(high nibble) BPB[3:0]: Back porch setting in partial mode. The minimum setting is 0x01.
						//(low nibble)  FPC[3:0]: Front porch setting in partial mode. The minimum setting is 0x01.

	// GCTRL (B7h): Gate Control
	ST7789_sendCommand(ST7789_B7_GCTRL);
	ST7789_sendData(0x35);

	//(high nibble) VGHS[2:0]: VGH Setting.
	// 0x0- => 12.20v
	// 0x1- => 12.54v
	// 0x2- => 12.89v
	// 0x3- => 13.26v <<<<<
	// 0x4- => 13.65v
	// 0x5- => 14.06v
	// 0x6- => 14.50v
	// 0x7- => 14.97v
	//(low nibble) VGLS[2:0]: VGL Setting.
	// 0x-0 =>  -7.16v
	// 0x-1 =>  -7.67v
	// 0x-2 =>  -8.23v
	// 0x-3 =>  -8.87v
	// 0x-4 =>  -9.60v
	// 0x-5 => -10.43v <<<<<
	// 0x-6 => -11.38v
	// 0x-7 => -12.50v

	// VCOMS (BBh): VCOM Setting
	ST7789_sendCommand(ST7789_BB_VCOMS);
	ST7789_sendData(0x2B);

	// VCOMS[5:0]
	// 0x00 => 0.100v
	// 0x01 => 0.125v
	// 0x02 => 0.150v
	// 0x03 => 0.175v
	// 0x04 => 0.200v
	// 0x05 => 0.225v
	// 0x06 => 0.250v
	// 0x07 => 0.275v
	// 0x08 => 0.300v
	// 0x09 => 0.325v
	// 0x0A => 0.350v
	// 0x0B => 0.375v
	// 0x0C => 0.400v
	// 0x0D => 0.425v
	// 0x0E => 0.450v
	// 0x0F => 0.475v
	// 0x10 => 0.500v
	// 0x11 => 0.525v
	// 0x12 => 0.550v
	// 0x13 => 0.575v
	// 0x14 => 0.600v
	// 0x15 => 0.625v
	// 0x16 => 0.650v
	// 0x17 => 0.675v
	// 0x18 => 0.700v
	// 0x19 => 0.725v
	// 0x1A => 0.750v
	// 0x1B => 0.775v
	// 0x1C => 0.800v <<<< V
	// 0x1D => 0.825v
	// 0x1E => 0.850v
	// 0x1F => 0.875v
	// 0x20 => 0.900v
	// 0x21 => 0.925v
	// 0x22 => 0.950v
	// 0x23 => 0.975v
	// 0x24 => 1.000v
	// 0x25 => 1.025v
	// 0x26 => 1.050v
	// 0x27 => 1.075v
	// 0x28 => 1.100v
	// 0x29 => 1.125v
	// 0x2A => 1.150v
	// 0x2B => 1.175v <<<< W
	// 0x2C => 1.200v
	// 0x2D => 1.225v
	// 0x2E => 1.250v
	// 0x2F => 1.275v
	// 0x30 => 1.300v
	// 0x31 => 1.325v
	// 0x32 => 1.350v
	// 0x33 => 1.375v
	// 0x34 => 1.400v
	// 0x35 => 1.425v <<<< X
	// 0x36 => 1.450v
	// 0x37 => 1.475v
	// 0x38 => 1.500v
	// 0x39 => 1.525v
	// 0x3A => 1.550v
	// 0x3B => 1.575v
	// 0x3C => 1.600v
	// 0x3D => 1.625v
	// 0x3E => 1.650v
	// 0x3F => 1.675v

	// LCMCTRL (C0h): LCM Control
	ST7789_sendCommand(ST7789_C0_LCMCTRL);
	ST7789_sendData(0x2C);
	// 0010 1100
	// |||| ||||--  GS: Gate scan inversion enable: 1: enable, 0: disable.
	// |||| |||---  XMX: XOR MX setting in command 36h.
	// |||| ||---- *XMV: XOR MV setting in command 36h.
	// |||| |----- *XMH: XOR RGB setting in command 36h, in RGB interface
	// ||||             without RAM mode can support column address order.
	// ||||-------  XREV: XOR inverse setting in command 21h.
	// |||-------- *XBGR: XOR RGB setting in command 36h.
	// ||---------  XMY: XOR MY setting in command 36h.
	// |----------  Unused: 0

	// VDVVRHEN (C2h): VDV and VRH Command Enable
	ST7789_sendCommand(ST7789_C2_VDVVRHEN);
	ST7789_sendData(0x01); // CMDEN=”0”: VDV and VRH register value comes from NVM.
						// CMDEN=”1”, VDV and VRH register value comes from command write.
	ST7789_sendData(0xFF);

	// VRHS (C3h): VRH Set
	ST7789_sendCommand(ST7789_C3_VRHS);
	ST7789_sendData(0x20);

	// 0x00 => VAP(GVDD)(V) = +3.55 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) = -3.55 + (vcom + vcom_offset - 0.5vdv)
	// 0x01 => VAP(GVDD)(V) = +3.60 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) = -3.60 + (vcom + vcom_offset - 0.5vdv)
	// 0x02 => VAP(GVDD)(V) = +3.65 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) = -3.65 + (vcom + vcom_offset - 0.5vdv)
	// 0x03 => VAP(GVDD)(V) = +3.70 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 3.70 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x04 => VAP(GVDD)(V) = +3.75 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 3.75 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x05 => VAP(GVDD)(V) = +3.80 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 3.80 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x06 => VAP(GVDD)(V) = +3.85 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 3.85 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x07 => VAP(GVDD)(V) = +3.90 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 3.90 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x08 => VAP(GVDD)(V) = +3.95 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 3.95 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x09 => VAP(GVDD)(V) = +4.00 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.00 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x0A => VAP(GVDD)(V) = +4.05 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.05 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x0B => VAP(GVDD)(V) = +4.10 + (vcom + vcom_offset + 0.5vdv) <<<< V
	//         VAN(GVCL)(V) =- 4.10 + (vcom + vcom_offset-+ 0.5vdv) <<<< V
	// 0x0C => VAP(GVDD)(V) = +4.15 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.15 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x0D => VAP(GVDD)(V) = +4.20 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.20 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x0E => VAP(GVDD)(V) = +4.25 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.25 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x0F => VAP(GVDD)(V) = +4.30 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.30 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x10 => VAP(GVDD)(V) = +4.35 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.35 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x11 => VAP(GVDD)(V) = +4.40 + (vcom + vcom_offset + 0.5vdv) <<<< X
	//         VAN(GVCL)(V) =- 4.40 + (vcom + vcom_offset-+ 0.5vdv) <<<< X
	// 0x12 => VAP(GVDD)(V) = +4.45 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.45 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x13 => VAP(GVDD)(V) = +4.50 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.50 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x14 => VAP(GVDD)(V) = +4.55 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.55 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x15 => VAP(GVDD)(V) = +4.60 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.60 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x16 => VAP(GVDD)(V) = +4.65 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.65 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x17 => VAP(GVDD)(V) = +4.70 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.70 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x18 => VAP(GVDD)(V) = +4.75 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.75 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x19 => VAP(GVDD)(V) = +4.80 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.80 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x1A => VAP(GVDD)(V) = +4.85 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.85 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x1B => VAP(GVDD)(V) = +4.90 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.90 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x1C => VAP(GVDD)(V) = +4.95 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 4.95 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x1D => VAP(GVDD)(V) = +5.00 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.00 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x1E => VAP(GVDD)(V) = +5.05 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.05 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x1F => VAP(GVDD)(V) = +5.10 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.10 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x20 => VAP(GVDD)(V) = +5.15 + (vcom + vcom_offset + 0.5vdv) <<<< W
	//         VAN(GVCL)(V) =- 5.15 + (vcom + vcom_offset-+ 0.5vdv) <<<< W
	// 0x21 => VAP(GVDD)(V) = +5.20 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.20 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x22 => VAP(GVDD)(V) = +5.25 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.25 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x23 => VAP(GVDD)(V) = +5.30 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.30 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x24 => VAP(GVDD)(V) = +5.35 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.35 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x25 => VAP(GVDD)(V) = +5.40 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.40 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x26 => VAP(GVDD)(V) = +5.45 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.45 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x27 => VAP(GVDD)(V) = +5.50 + (vcom + vcom_offset + 0.5vdv)
	//         VAN(GVCL)(V) =- 5.50 + (vcom + vcom_offset-+ 0.5vdv)
	// 0x28~0x3Fh => Reserved

	//VDVS (C4h): VDV Set
	ST7789_sendCommand(ST7789_C4_VDVSET);
	ST7789_sendData(0x20);
	// 0x00 => -0.800
	// 0x01 => -0.775
	// 0x02 => -0.750
	// 0x03 => -0.725
	// 0x04 => -0.700
	// 0x05 => -0.675
	// 0x06 => -0.650
	// 0x07 => -0.625
	// 0x08 => -0.600
	// 0x09 => -0.575
	// 0x0A => -0.550
	// 0x0B => -0.525
	// 0x0C => -0.500
	// 0x0D => -0.475
	// 0x0E => -0.450
	// 0x0F => -0.425
	// 0x10 => -0.400
	// 0x11 => -0.375
	// 0x12 => -0.350
	// 0x13 => -0.325
	// 0x14 => -0.300
	// 0x15 => -0.275
	// 0x16 => -0.250
	// 0x17 => -0.225
	// 0x18 => -0.200
	// 0x19 => -0.175
	// 0x1A => -0.150
	// 0x1B => -0.125
	// 0x1C => -0.100
	// 0x1D => -0.075
	// 0x1E => -0.050
	// 0x1F => -0.025
	// 0x20 => +0.000 <<<<<
	// 0x21 => +0.025
	// 0x22 => +0.050
	// 0x23 => +0.075
	// 0x24 => +0.100
	// 0x25 => +0.125
	// 0x26 => +0.150
	// 0x27 => +0.175
	// 0x28 => +0.200
	// 0x29 => +0.225
	// 0x2A => +0.250
	// 0x2B => +0.275
	// 0x2C => +0.300
	// 0x2D => +0.325
	// 0x2E => +0.350
	// 0x2F => +0.375
	// 0x30 => +0.400
	// 0x31 => +0.425
	// 0x32 => +0.450
	// 0x33 => +0.475
	// 0x34 => +0.500
	// 0x35 => +0.525
	// 0x36 => +0.550
	// 0x37 => +0.575
	// 0x38 => +0.600
	// 0x39 => +0.625
	// 0x3A => +0.650
	// 0x3B => +0.675
	// 0x3C => +0.700
	// 0x3D => +0.725
	// 0x3E => +0.750
	// 0x3F => +0.775

	//FRCTRL2 (C6h): Frame Rate Control in Normal Mode
	ST7789_sendCommand(ST7789_C6_FRCTR2);
	ST7789_sendData(0x0F);
	// 0000 1111
	// ||||-||||--  RTNA[4:0]
	// |||--------  NLA[2:0] : Inversion selection in normal mode
	//                000 = dot inversion
	//                111 = column inversion
	// RTNA[4:0] => FR in normal mode (Hz)
	// 0x00 => 119 Hz
	// 0x01 => 111 Hz
	// 0x02 => 105 Hz
	// 0x03 =>  99 Hz
	// 0x04 =>  94 Hz
	// 0x05 =>  90 Hz
	// 0x06 =>  86 Hz
	// 0x07 =>  82 Hz
	// 0x08 =>  78 Hz
	// 0x09 =>  75 Hz
	// 0x0A =>  72 Hz
	// 0x0B =>  69 Hz
	// 0x0C =>  67 Hz
	// 0x0D =>  64 Hz
	// 0x0E =>  62 Hz
	// 0x0F =>  60 Hz <<<<<
	// 0x10 =>  58 Hz
	// 0x11 =>  57 Hz
	// 0x12 =>  55 Hz
	// 0x13 =>  53 Hz
	// 0x14 =>  52 Hz
	// 0x15 =>  50 Hz
	// 0x16 =>  49 Hz
	// 0x17 =>  48 Hz
	// 0x18 =>  46 Hz
	// 0x19 =>  45 Hz
	// 0x1A =>  44 Hz
	// 0x1B =>  43 Hz
	// 0x1C =>  42 Hz
	// 0x1D =>  41 Hz
	// 0x1E =>  40 Hz
	// 0x1F =>  39 Hz

	// PWCTRL1 (D0h): Power Control 1
	ST7789_sendCommand(ST7789_D0_PWCTRL1);
	ST7789_sendData(0xA4); //Fixed vector
	ST7789_sendData(0xA1); //AVDD=6.8V, AVCL=-4.8V, VDS=2.3V
						// DDCC --VV
						// |||| ||||--  VDS[1:0]:
						// ||||           00 = 2.19v
						// ||||           01 = 2.30v <<<<<
						// ||||           10 = 2.40v
						// ||||           11 = 2.51v
						// ||||-------  AVCL[1:0]:
						// ||             00 = -4.4v
						// ||             01 = -4.6v
						// ||             10 = -4.8v <<<<<
						// ||             11 = -5.0v
						// ||---------  AVDD[1:0]:
						//                00 = 6.4v
						//                01 = 6.6v
						//                10 = 6.8v <<<<<
						//                11 = Reserved

	// PVGAMCTRL (E0h): Positive Voltage Gamma Control
	ST7789_sendCommand(ST7789_E0_PVGAMCTRL);
	// Pile of magic numbers :-(
	ST7789_sendData(0xD0);
	ST7789_sendData(0xCA);
	ST7789_sendData(0x0E);
	ST7789_sendData(0x08);
	ST7789_sendData(0x09);
	ST7789_sendData(0x07);
	ST7789_sendData(0x2D);
	ST7789_sendData(0x3B);
	ST7789_sendData(0x3D);
	ST7789_sendData(0x34);
	ST7789_sendData(0x0A);
	ST7789_sendData(0x0A);
	ST7789_sendData(0x1B);
	ST7789_sendData(0x28);

	// NVGAMCTRL (E1h): Negative Voltage Gamma Control
	ST7789_sendCommand(ST7789_E1_NVGAMCTRL);

	// Pile of magic numbers :-(
	ST7789_sendData(0xD0);
	ST7789_sendData(0xCA);
	ST7789_sendData(0x0F);
	ST7789_sendData(0x08);
	ST7789_sendData(0x08);
	ST7789_sendData(0x07);
	ST7789_sendData(0x2E);
	ST7789_sendData(0x5C);
	ST7789_sendData(0x40);
	ST7789_sendData(0x34);
	ST7789_sendData(0x09);
	ST7789_sendData(0x0B);
	ST7789_sendData(0x1B);
	ST7789_sendData(0x28);

	// INVON (21h): Display Inversion On (correct for W)
	ST7789_sendCommand(ST7789_21_INVON);

	// CASET (2Ah): Column Address Set
	ST7789_sendCommand(ST7789_2A_CASET);
	ST7789_sendData(0x00); //Start MSB Start = 0
	ST7789_sendData(0x00); //Start LSB
	ST7789_sendData(0x00); //End MSB End = 249
	ST7789_sendData(0xEF); //End LSB

	// RASET (2Bh): Row Address Set
	ST7789_sendCommand(ST7789_2B_RASET);
	ST7789_sendData(0x00); //Start MSB Start = 0
	ST7789_sendData(0x00); //Start LSB
	ST7789_sendData(0x01); //End MSB End = 319
	ST7789_sendData(0x3F); //End LSB

	// DISPON (29h): Display On
	ST7789_sendCommand(ST7789_29_DISPON);
	delay(1);
  
  ST7789_RGBMode();
}
//==============================================================================
void ST7789_RGBMode(void)
{
	//put panel into RGB mode for RT811
	ST7789_sendCommand(ST7789_28_DISPOFF);
	//ST7789_RAMCTRL defaults to 00/F0
	ST7789_sendCommand(ST7789_B0_RAMCTRL);
	ST7789_sendData(0x11); //11 = RGB interface
	ST7789_sendData(0xF0); //F0 = 18bit
	//on
	ST7789_sendCommand(ST7789_29_DISPON);
	delay(1);
}